@@ -3,7 +3,7 @@
 #
 # This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
-# https://gitlab.com/tsfpga/tsfpga
+# https://github.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
 # First party libraries
@@ -11,7 +11,9 @@ from tsfpga.module import BaseModule
 
 
 class Module(BaseModule):
-    def get_simulation_files(self, include_ip_cores, **kwargs):  # pylint: disable=arguments-differ
+    def get_simulation_files(  # type: ignore  # pylint: disable=arguments-differ
+        self, include_ip_cores: bool, **kwargs
+    ):
         """
         Exclude files that depend on IP cores, if instructed to by the simulation script.
         """
